@@ -28,6 +28,15 @@ use std::path::PathBuf;
 
 pub type ModulePath = Vec<String>;
 
+pub trait Plugin: std::fmt::Debug {
+    fn generate(
+        &self,
+        module_ctxt: &mut context::ModuleContext,
+        definition_name: &str,
+        definition: &Definition,
+    );
+}
+
 #[derive(Debug)]
 pub struct CompilerConfig {
     pub bundle: Map<ModulePath, Schema>,
@@ -35,6 +44,7 @@ pub struct CompilerConfig {
     pub fully_qualified_module_prefix: String,
     pub support_crate: String,
     pub module_aliases: Map<ModulePath, String>,
+    pub plugins: Vec<Box<dyn Plugin>>,
 }
 
 impl CompilerConfig {
@@ -48,6 +58,12 @@ impl CompilerConfig {
             fully_qualified_module_prefix: fully_qualified_module_prefix,
             support_crate: "preserves_schema".to_owned(),
             module_aliases: Map::new(),
+            plugins: vec![
+                Box::new(types::TypePlugin),
+                Box::new(readers::ReaderPlugin),
+                Box::new(parsers::ParserPlugin),
+                Box::new(unparsers::UnparserPlugin),
+            ],
         }
     }
 
@@ -136,15 +152,16 @@ pub fn compile(config: &CompilerConfig) -> io::Result<()> {
         let modes = modes; // rebind as non-mutable
 
         for (n, d) in &v.definitions.0 {
-            m.define_type(item(types::render_definition_type(&m, n, &types::definition_type(d))));
-            m.define_type(item(seq!["impl preserves::value::Domain for ",
-                                    names::render_constructor(n), " {}"]));
-            for mode in &modes {
-                m.mode = *mode;
-                parsers::gen_definition_parser(&mut m, n, d);
-                unparsers::gen_definition_unparser(&mut m, n, d);
+            for plugin in config.plugins.iter() {
+                plugin.generate(&mut m, n, d);
             }
-            readers::gen_definition_reader(&mut m, n, d);
+            for mode in &modes {
+                m.mode = Some(*mode);
+                for plugin in config.plugins.iter() {
+                    plugin.generate(&mut m, n, d);
+                }
+            }
+            m.mode = None;
         }
 
         //---------------------------------------------------------------------------
@@ -162,7 +179,7 @@ pub fn compile(config: &CompilerConfig) -> io::Result<()> {
         lines.push("".to_owned());
 
         for mode in &modes {
-            m.mode = *mode;
+            m.mode = Some(*mode);
             lines.push(format!("mod {} {{", m.target_prefix()));
             lines.push("    use super::_Any;".to_owned());
             lines.push(format!("    use {}::support as _support;", &config.support_crate));
@@ -180,6 +197,7 @@ pub fn compile(config: &CompilerConfig) -> io::Result<()> {
             lines.push("}".to_owned());
             lines.push("".to_owned());
         }
+        m.mode = None;
 
         for i in m.typedefs {
             lines.push(Formatter::to_string(i));
